'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _express8 = require('express');

var _express9 = _interopRequireDefault(_express8);

var _knex = require('knex');

var _knex2 = _interopRequireDefault(_knex);

var _apolloServer = require('apollo-server');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _expressJwt = require('express-jwt');

var _expressJwt2 = _interopRequireDefault(_expressJwt);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Lado
 *
 * @constructor
 */

var Lado = function () {
	function Lado() {
		_classCallCheck(this, Lado);

		this.config = _config2.default;
		this.registry = {};
		this.express = (0, _express9.default)();
	}

	_createClass(Lado, [{
		key: 'configure',
		value: function configure(conf) {
			if (_lodash2.default.isString(conf)) {
				//load from directory
				this.config.loadFromFolder(conf);
			} else if (_lodash2.default.isObject(conf)) {
				//set from raw object
				this.config.loadFromObj(conf);
			} else {
				throw new Error('Invalid configuration settings');
			}

			this.configureDb();
			this.configureExpress();
		}

		/**
   **************************************************
   * Registry Methods
   **************************************************
   */

	}, {
		key: 'make',
		value: function make(key) {
			return this.registry[key];
		}
	}, {
		key: 'register',
		value: function register(key, inst) {
			this.registry[key] = inst;
		}

		/**
   **************************************************
   * Database Methods
   **************************************************
   */

	}, {
		key: 'db',
		value: function db() {
			throw new Exception('Database not configured');
		}
	}, {
		key: 'configureDb',
		value: function configureDb() {
			this.db = (0, _knex2.default)({
				client: this.config.get('database.client'),
				debug: this.config.get('database.debug'),
				connection: {
					host: this.config.get('database.host'),
					user: this.config.get('database.user'),
					password: this.config.get('database.password'),
					database: this.config.get('database.database'),
					charset: this.config.get('database.charset', 'utf8')
				}
			});
		}

		/**
   **************************************************
   * Express Methods
   **************************************************
   */

	}, {
		key: 'configureExpress',
		value: function configureExpress() {
			var VIEW_ENGINE = this.config.get('views.engine');
			var VIEW_PATH = this.config.get('views.path');
			var STATIC = this.config.get('static.path');
			var SECRET = this.config.get('auth.secret', 'ssshhhh');

			VIEW_ENGINE && this.express.set('view engine', VIEW_ENGINE);
			VIEW_PATH && this.express.set('views', VIEW_PATH);
			STATIC && this.express.use(_express9.default.static(STATIC));

			this.express.use((0, _expressJwt2.default)({
				secret: SECRET,
				credentialsRequired: false
			}));
		}
	}, {
		key: 'listen',
		value: function listen() {
			var NAME = this.config.get('app.name', 'Lado');
			var HOST = this.config.get('app.host');
			var PORT = this.config.get('app.port');

			this.express.listen(PORT, function () {
				return console.log(NAME + ' server is now running on http://' + HOST + ':' + PORT);
			});
		}
	}, {
		key: 'set',
		value: function set() {
			var _express;

			(_express = this.express).set.apply(_express, arguments);
		}
	}, {
		key: 'get',
		value: function get() {
			var _express2;

			(_express2 = this.express).get.apply(_express2, arguments);
		}
	}, {
		key: 'use',
		value: function use() {
			var _express3;

			(_express3 = this.express).use.apply(_express3, arguments);
		}
	}, {
		key: 'post',
		value: function post() {
			var _express4;

			(_express4 = this.express).post.apply(_express4, arguments);
		}
	}, {
		key: 'put',
		value: function put() {
			var _express5;

			(_express5 = this.express).put.apply(_express5, arguments);
		}
	}, {
		key: 'del',
		value: function del() {
			var _express6;

			(_express6 = this.express).del.apply(_express6, arguments);
		}
	}, {
		key: 'head',
		value: function head() {
			var _express7;

			(_express7 = this.express).head.apply(_express7, arguments);
		}
	}, {
		key: 'resource',
		value: function resource(mount, Controller) {}

		/**
   **************************************************
   * Apollo Methods
   **************************************************
   */

	}, {
		key: 'graphMiddleware',
		value: function graphMiddleware() {
			var CONFIG = this.config.get('graphql');

			return (0, _apolloServer.apolloServer)(function (req) {
				var conf = {
					graphiql: CONFIG.graphiql,
					pretty: CONFIG.pretty,
					schema: CONFIG.schema,
					context: req.user
				};

				// TODO read this from an environment variable
				if (false) {
					conf.mocks = CONFIG.mocks;
				} else {
					conf.resolvers = CONFIG.resolvers;
				}

				return conf;
			});
		}
	}]);

	return Lado;
}();

exports.default = new Lado();